@@ -76,6 +76,11 @@
 
 #define UID_LENGTH 4
 
+#define TYPE1_TAG_VER_1_1 0x11
+#define TYPE1_TAG_STATIC_SIZE_120 0x0E
+#define TYPE1_READ_WRITE_ACCESS 0x00
+#define TYPE1_STATIC_TAG_DATA_LENGTH 116
+
 struct type1_cmd {
 	uint8_t cmd;
 	uint8_t addr;
@@ -103,6 +108,7 @@ struct t1_cookie {
 	uint32_t current_byte;  /* Static tag */
 	struct near_ndef_message *ndef;
 	near_tag_io_cb cb;
+	uint8_t cc[LEN_CC_BYTES];
 };
 
 static void t1_init_cmd(struct type1_tag *tag, struct type1_cmd *cmd)
@@ -113,10 +119,17 @@ static void t1_init_cmd(struct type1_tag *tag, struct type1_cmd *cmd)
 	memcpy(cmd->uid, tag->uid, UID_LENGTH);
 }
 
-static void t1_cookie_release(struct t1_cookie *cookie)
+static int t1_cookie_release(int err, void *data)
 {
+	struct t1_cookie *cookie = data;
+
+	DBG("%p", cookie);
+
 	if (cookie == NULL)
-		return;
+		return err;
+
+	if (cookie->cb != NULL)
+		cookie->cb(cookie->adapter_idx, cookie->target_idx, err);
 
 	if (cookie->ndef)
 		g_free(cookie->ndef->data);
@@ -124,30 +137,28 @@ static void t1_cookie_release(struct t1_cookie *cookie)
 	g_free(cookie->ndef);
 	g_free(cookie);
 	cookie = NULL;
+
+	return err;
 }
 
-/* Read segments (128 bytes)and store them to the tag data block */
-static int segment_read_recv(uint8_t *resp, int length, void *data)
+/* Read segments (128 bytes) and store them to the tag data block */
+static int data_recv(uint8_t *resp, int length, void *data)
 {
 	struct type1_tag *t1_tag = data;
 	struct type1_cmd t1_cmd;
 	uint8_t *tagdata;
 	size_t data_length;
 
-	int err;
-
 	DBG("%d", length);
 
-	if (length < 0) {
-		err = length;
-		goto out_err;
-	}
+	if (length < 0)
+		return length;
 
 	length = length - LEN_STATUS_BYTE;  /* ignore first byte */
 
-	/* Add data to tag mem*/
+	/* Add data to tag mem */
 	tagdata = near_tag_get_data(t1_tag->tag, &data_length);
-	memcpy(tagdata + t1_tag->data_read, resp+1, length);
+	memcpy(tagdata + t1_tag->data_read, resp + 1, length);
 
 	/* Next segment */
 	t1_tag->data_read =  t1_tag->data_read + length;
@@ -160,11 +171,9 @@ static int segment_read_recv(uint8_t *resp, int length, void *data)
 		t1_cmd.cmd = CMD_READ_SEGS;
 		t1_cmd.addr = (t1_tag->current_seg << 4) & 0xFF;
 
-		err = near_adapter_send(t1_tag->adapter_idx,
-				(uint8_t *)&t1_cmd, sizeof(t1_cmd),
-				segment_read_recv, t1_tag);
-		if (err < 0)
-			goto out_err;
+		return near_adapter_send(t1_tag->adapter_idx,
+				(uint8_t *) &t1_cmd, sizeof(t1_cmd),
+				data_recv, t1_tag, NULL);
 	} else { /* This is the end */
 		GList *records;
 
@@ -173,17 +182,15 @@ static int segment_read_recv(uint8_t *resp, int length, void *data)
 		records = near_tlv_parse(tagdata, data_length);
 		near_tag_add_records(t1_tag->tag, records, t1_tag->cb, 0);
 
-		err = 0;
-
 		/* free memory */
 		g_free(t1_tag);
-	}
 
-out_err:
-	return err;
+		return 0;
+	}
 }
 
-/* The dynamic read function:
+/*
+ * The dynamic read function:
  * Bytes [0..3] : CC
  * [4..8]: TLV Lock ControlIT (0x01, 0x03, v1, V2, V3)
  * [9..13]: TLV Reserved Memory Control	(0x02, 0x03, V1, V2, V3)
@@ -207,9 +214,10 @@ static int read_dynamic_tag(uint8_t *cc, int length, void *data)
 	pndef = pndef + 5;	/* skip TLV Lock bits bytes */
 	pndef = pndef + 5;	/* skip TLV ControlIT bytes */
 
-	/* Save first NFC bytes to tag memory
+	/*
+	 * Save first NFC bytes to tag memory
 	 * 10 blocks[0x3..0xC] of 8 bytes + 2 bytes from block 2
-	 * */
+	 */
 	memcpy(tagdata,	pndef, 10 * BLOCK_SIZE + 2);
 
 	/* Read the next one, up to the end of the data area */
@@ -226,14 +234,14 @@ static int read_dynamic_tag(uint8_t *cc, int length, void *data)
 
 	return near_adapter_send(t1_tag->adapter_idx,
 			(uint8_t *)&t1_cmd, sizeof(t1_cmd),
-			segment_read_recv, t1_tag);
+			data_recv, t1_tag, NULL);
 }
 
 static int meta_recv(uint8_t *resp, int length, void *data)
 {
 	struct t1_cookie *cookie = data;
 	struct near_tag *tag;
-	struct type1_tag *t1_tag = NULL;
+	struct type1_tag *t1_tag;
 
 	uint8_t *cc;
 	int err = -EOPNOTSUPP;
@@ -247,7 +255,7 @@ static int meta_recv(uint8_t *resp, int length, void *data)
 
 	/* First byte is cmd status */
 	if (resp[OFFSET_STATUS_CMD] != 0) {
-		DBG("Command failed: 0x%x",resp[OFFSET_STATUS_CMD]);
+		DBG("Command failed: 0x%x", resp[OFFSET_STATUS_CMD]);
 		err = -EIO;
 		goto out_err;
 	}
@@ -255,14 +263,22 @@ static int meta_recv(uint8_t *resp, int length, void *data)
 	/* Check Magic NFC tag */
 	cc = TAG_T1_DATA_CC(resp);
 	if (TAG_T1_DATA_NFC(cc) == 0) {
-		DBG("Not a valid NFC magic tag: 0x%x",cc[0]);
-		err = -EINVAL;
-		goto out_err;
+		if (resp[OFFSET_HEADER_ROM] == HR0_TYPE1_STATIC) {
+			err = near_tag_add_data(cookie->adapter_idx,
+						cookie->target_idx,
+						NULL,
+						TYPE1_STATIC_TAG_DATA_LENGTH);
+		} else {
+			near_error("Not a valid NFC magic tag 0x%x", cc[0]);
+			err = -EINVAL;
+			goto out_err;
+		}
+	} else {
+		/* Add data to the tag */
+		err = near_tag_add_data(cookie->adapter_idx, cookie->target_idx,
+						NULL, TAG_T1_DATA_LENGTH(cc));
 	}
 
-	/* Add data to the tag */
-	err = near_tag_add_data(cookie->adapter_idx, cookie->target_idx,
-					NULL, TAG_T1_DATA_LENGTH(cc));
 	if (err < 0)
 		goto out_err;
 
@@ -283,7 +299,7 @@ static int meta_recv(uint8_t *resp, int length, void *data)
 	t1_tag->tag = tag;
 	memcpy(t1_tag->uid, cookie->uid, UID_LENGTH);
 
-	/*s Set the ReadWrite flag */
+	/* Set the ReadWrite flag */
 	if (TAG_T1_WRITE_FLAG(cc) == TYPE1_NOWRITE_ACCESS)
 		near_tag_set_ro(tag, TRUE);
 	else
@@ -295,6 +311,11 @@ static int meta_recv(uint8_t *resp, int length, void *data)
 		size_t data_length;
 		GList *records;
 
+		if (TAG_T1_DATA_NFC(cc) == 0)
+			near_tag_set_blank(tag, TRUE);
+		else
+			near_tag_set_blank(tag, FALSE);
+
 		DBG("READ Static complete");
 
 		tagdata = near_tag_get_data(t1_tag->tag, &data_length);
@@ -311,19 +332,22 @@ static int meta_recv(uint8_t *resp, int length, void *data)
 	} else if ((resp[OFFSET_HEADER_ROM] & 0xF0) == HR0_TYPE2_HIGH) {
 		near_tag_set_memory_layout(tag, NEAR_TAG_MEMORY_DYNAMIC);
 		err = read_dynamic_tag(cc, length, t1_tag);
+		/*
+		 * As reading isn't complete,
+		 * callback shouldn't be called while freeing the cookie
+		 */
+		cookie->cb = NULL;
 	} else {
-		err = -EOPNOTSUPP ;
+		err = -EOPNOTSUPP;
 	}
 
+	if (err < 0)
+		g_free(t1_tag);
+
 out_err:
 	DBG("err %d", err);
 
-	if (err < 0 && cookie->cb)
-		cookie->cb(cookie->adapter_idx, cookie->target_idx, err);
-
-	t1_cookie_release(cookie);
-
-	return err;
+	return t1_cookie_release(err, cookie);
 }
 
 /*
@@ -342,7 +366,7 @@ static int rid_resp(uint8_t *resp, int length, void *data)
 
 	/* First byte is cmd status */
 	if (resp[OFFSET_STATUS_CMD] != 0) {
-		DBG("Command failed: 0x%x",resp[OFFSET_STATUS_CMD]);
+		DBG("Command failed: 0x%x", resp[OFFSET_STATUS_CMD]);
 		err = -EIO;
 		goto out_err;
 	}
@@ -363,69 +387,76 @@ static int rid_resp(uint8_t *resp, int length, void *data)
 
 	return near_adapter_send(cookie->adapter_idx,
 				(uint8_t *)&t1_cmd, sizeof(t1_cmd),
-				meta_recv, cookie);
+				meta_recv, cookie, t1_cookie_release);
 
 out_err:
 	DBG("err %d", err);
 
-	if (err < 0 && cookie->cb)
-		cookie->cb(cookie->adapter_idx, cookie->target_idx, err);
+	return t1_cookie_release(err, cookie);
+}
 
-	t1_cookie_release(cookie);
+static int nfctype1_read_meta(uint32_t adapter_idx, uint32_t target_idx,
+						near_tag_io_cb cb, uint8_t *uid)
+{
+	struct type1_cmd cmd;
+	struct t1_cookie *cookie;
 
-	return err;
+	DBG("");
+
+	memset(&cmd, 0, sizeof(cmd));
+
+	cookie = g_try_malloc0(sizeof(struct t1_cookie));
+	if (cookie == NULL)
+		return -ENOMEM;
+
+	cookie->adapter_idx = adapter_idx;
+	cookie->target_idx = target_idx;
+	cookie->cb = cb;
+
+	if (uid != NULL) {
+		cmd.cmd = CMD_READ_ALL; /* Read ALL cmd give 124 bytes */
+		memcpy(cmd.uid, uid, UID_LENGTH);
+		memcpy(cookie->uid, uid, UID_LENGTH);
+
+		return near_adapter_send(adapter_idx, (uint8_t *) &cmd,
+						sizeof(cmd), meta_recv, cookie,
+						t1_cookie_release);
+	} else {
+		cmd.cmd = CMD_RID;
+
+		return near_adapter_send(adapter_idx, (uint8_t *) &cmd,
+						sizeof(cmd), rid_resp, cookie,
+						t1_cookie_release);
+	}
 }
 
 /* First step: RID to get the tag UID */
 static int nfctype1_read(uint32_t adapter_idx,
 				uint32_t target_idx, near_tag_io_cb cb)
 {
-	struct type1_cmd t1_cmd;
-	struct t1_cookie *cookie;
-	uint8_t *uid, uid_length;
+	uint8_t *uid;
+	uint8_t  uid_length;
+	int err;
 
 	DBG("");
 
 	uid = near_tag_get_nfcid(adapter_idx, target_idx, &uid_length);
 	if (uid == NULL || uid_length != UID_LENGTH) {
-		if (uid != NULL && uid_length != UID_LENGTH) {
+		if (uid != NULL) {
 			near_error("Invalid UID");
 
 			g_free(uid);
 			return -EINVAL;
 		}
 
-		t1_cmd.cmd = CMD_RID;
-		t1_cmd.addr = 0;
-		t1_cmd.data[0] = 0;
-		memset(t1_cmd.uid, 0, UID_LENGTH);
-
-		cookie = g_try_malloc0(sizeof(struct t1_cookie));
-		cookie->adapter_idx = adapter_idx;
-		cookie->target_idx = target_idx;
-		cookie->cb = cb;
-
-		return near_adapter_send(adapter_idx,
-					(uint8_t *)&t1_cmd, sizeof(t1_cmd),
-					rid_resp, cookie);
+		return nfctype1_read_meta(adapter_idx, target_idx, cb, NULL);
 	}
 
-	t1_cmd.cmd = CMD_READ_ALL;     /* Read ALL cmd give 124 bytes */
-	t1_cmd.addr = 0;	       /* NA */
-	t1_cmd.data[0] = 0;
-	memcpy(t1_cmd.uid, uid, UID_LENGTH);
-
-	cookie = g_try_malloc0(sizeof(struct t1_cookie));
-	cookie->adapter_idx = adapter_idx;
-	cookie->target_idx = target_idx;
-	memcpy(cookie->uid, uid, UID_LENGTH);
-	cookie->cb = cb;
+	err = nfctype1_read_meta(adapter_idx, target_idx, cb, uid);
 
 	g_free(uid);
 
-	return near_adapter_send(adapter_idx, (uint8_t *)&t1_cmd, sizeof(t1_cmd),
-							meta_recv, cookie);
-
+	return err;
 }
 
 static int write_nmn_e1_resp(uint8_t *resp, int length, void *data)
@@ -443,11 +474,7 @@ static int write_nmn_e1_resp(uint8_t *resp, int length, void *data)
 
 	DBG("Done writing");
 
-	cookie->cb(cookie->adapter_idx, cookie->target_idx, err);
-
-	t1_cookie_release(cookie);
-
-	return err;
+	return t1_cookie_release(err, cookie);
 }
 
 static int write_nmn_e1(struct t1_cookie *cookie)
@@ -461,11 +488,12 @@ static int write_nmn_e1(struct t1_cookie *cookie)
 	cmd.data[0] = TYPE1_MAGIC;
 	memcpy(cmd.uid, cookie->uid, UID_LENGTH);
 
-	return near_adapter_send(cookie->adapter_idx, (uint8_t *)&cmd,
-					sizeof(cmd), write_nmn_e1_resp, cookie);
+	return near_adapter_send(cookie->adapter_idx, (uint8_t *) &cmd,
+					sizeof(cmd), write_nmn_e1_resp, cookie,
+					t1_cookie_release);
 }
 
-static int data_write(uint8_t *resp, int length, void *data)
+static int data_write_resp(uint8_t *resp, int length, void *data)
 {
 	struct t1_cookie *cookie = data;
 	uint8_t addr = 0;
@@ -476,12 +504,12 @@ static int data_write(uint8_t *resp, int length, void *data)
 
 	if (length < 0) {
 		err = length;
-		goto out;
+		goto out_err;
 	}
 
 	if (resp[OFFSET_STATUS_CMD] != 0) {
 		err = -EIO;
-		goto out;
+		goto out_err;
 	}
 
 	if (cookie->ndef->offset > cookie->ndef->length)
@@ -500,23 +528,19 @@ static int data_write(uint8_t *resp, int length, void *data)
 	cookie->ndef->offset++;
 	cookie->current_byte++;
 
-	err = near_adapter_send(cookie->adapter_idx, (uint8_t *)&cmd,
-					sizeof(cmd), data_write, cookie);
+	err = near_adapter_send(cookie->adapter_idx, (uint8_t *) &cmd,
+					sizeof(cmd), data_write_resp, cookie,
+					NULL);
 	if (err < 0)
-		goto out;
+		goto out_err;
 
 	return 0;
 
-out:
-	if (err < 0 && cookie->cb)
-		cookie->cb(cookie->adapter_idx, cookie->target_idx, err);
-
-	t1_cookie_release(cookie);
-
-	return err;
+out_err:
+	return t1_cookie_release(err, cookie);
 }
 
-static int write_nmn_0(uint32_t adapter_idx, uint32_t target_idx,
+static int data_write(uint32_t adapter_idx, uint32_t target_idx,
 			struct near_ndef_message *ndef, near_tag_io_cb cb)
 {
 	int err;
@@ -529,7 +553,8 @@ static int write_nmn_0(uint32_t adapter_idx, uint32_t target_idx,
 	uid = near_tag_get_nfcid(adapter_idx, target_idx, &uid_length);
 	if (uid == NULL || uid_length != UID_LENGTH) {
 		near_error("Invalid type 1 UID");
-		return -EINVAL;
+		err = -EINVAL;
+		goto out_err;
 	}
 
 	cmd.cmd  = CMD_WRITE_E;
@@ -541,7 +566,7 @@ static int write_nmn_0(uint32_t adapter_idx, uint32_t target_idx,
 	if (cookie == NULL) {
 		g_free(uid);
 		err = -ENOMEM;
-		return err;
+		goto out_err;
 	}
 
 	cookie->adapter_idx = adapter_idx;
@@ -554,15 +579,12 @@ static int write_nmn_0(uint32_t adapter_idx, uint32_t target_idx,
 
 	g_free(uid);
 
-	err = near_adapter_send(cookie->adapter_idx, (uint8_t *)&cmd,
-					sizeof(cmd), data_write, cookie);
-	if (err < 0)
-		goto out;
-
-	return 0;
-
-out:
-	t1_cookie_release(cookie);
+	return near_adapter_send(cookie->adapter_idx, (uint8_t *) &cmd,
+					sizeof(cmd), data_write_resp, cookie,
+					t1_cookie_release);
+out_err:
+	if (cb != NULL)
+		cb(adapter_idx, target_idx, err);
 
 	return err;
 }
@@ -582,27 +604,44 @@ static int nfctype1_write(uint32_t adapter_idx, uint32_t target_idx,
 				near_tag_io_cb cb)
 {
 	struct near_tag *tag;
+	int err;
 
 	DBG("");
 
-	if (ndef == NULL || cb == NULL)
-		return -EINVAL;
+	if (ndef == NULL || cb == NULL) {
+		err = -EINVAL;
+		goto out_err;
+	}
 
 	tag = near_tag_get_tag(adapter_idx, target_idx);
-	if (tag == NULL)
-		return -EINVAL;
+	if (tag == NULL) {
+		err = -EINVAL;
+		goto out_err;
+	}
 
 	/* This check is valid for only static tags.
-	 * Max data length on Type 2 Tag including TLV's
+	 * Max data length on Type 1 Tag including TLV's
 	 * is TYPE1_STATIC_MAX_DATA_SIZE */
 	if (near_tag_get_memory_layout(tag) == NEAR_TAG_MEMORY_STATIC) {
 		if ((ndef->length + 3) > TYPE1_STATIC_MAX_DATA_SIZE) {
-			near_error("not enough space on data");
-			return -ENOMEM;
+			near_error("not enough space on tag");
+			err = -ENOSPC;
+			goto out_err;
 		}
 	}
 
-	return write_nmn_0(adapter_idx, target_idx, ndef, cb);
+	return data_write(adapter_idx, target_idx, ndef, cb);
+
+	if (err < 0)
+		goto out_err;
+
+	return 0;
+
+out_err:
+	if (cb != NULL)
+		cb(adapter_idx, target_idx, err);
+
+	return err;
 }
 
 static int check_presence(uint8_t *resp, int length, void *data)
@@ -615,13 +654,7 @@ static int check_presence(uint8_t *resp, int length, void *data)
 	if (length < 0)
 		err = -EIO;
 
-	if (cookie->cb)
-		cookie->cb(cookie->adapter_idx,
-				cookie->target_idx, err);
-
-	t1_cookie_release(cookie);
-
-	return err;
+	return t1_cookie_release(err, cookie);
 }
 
 static int nfctype1_check_presence(uint32_t adapter_idx,
@@ -630,7 +663,6 @@ static int nfctype1_check_presence(uint32_t adapter_idx,
 	struct type1_cmd t1_cmd;
 	struct t1_cookie *cookie;
 	uint8_t *uid, uid_length;
-	int err;
 
 	DBG("");
 
@@ -655,15 +687,113 @@ static int nfctype1_check_presence(uint32_t adapter_idx,
 	cookie->target_idx = target_idx;
 	cookie->cb = cb;
 
-	err = near_adapter_send(adapter_idx, (uint8_t *)&t1_cmd, sizeof(t1_cmd),
-							check_presence, cookie);
+	return near_adapter_send(adapter_idx, (uint8_t *) &t1_cmd,
+				sizeof(t1_cmd), check_presence, cookie,
+				t1_cookie_release);
+}
+
+static int format_resp(uint8_t *resp, int length, void *data)
+{
+	int err = 0;
+	struct t1_cookie *cookie = data;
+	struct near_tag *tag;
+	struct type1_cmd cmd;
+	uint8_t addr;
+
+	DBG("");
+
+	if (length < 0 || resp[0] != 0) {
+		err = -EIO;
+		goto out_err;
+	}
+
+	if (cookie->current_byte < LEN_CC_BYTES) {
+		cmd.cmd  = CMD_WRITE_E;
+		addr = cookie->current_block << 3;
+		cmd.addr = addr | (cookie->current_byte & 0x7);
+		cmd.data[0] = cookie->cc[cookie->current_byte];
+		cookie->current_byte++;
+		memcpy(cmd.uid, cookie->uid, UID_LENGTH);
+
+		return near_adapter_send(cookie->adapter_idx, (uint8_t *) &cmd,
+					sizeof(cmd), format_resp, cookie,
+					t1_cookie_release);
+	} else {
+		tag = near_tag_get_tag(cookie->adapter_idx, cookie->target_idx);
+		if (tag == NULL) {
+			err = -EINVAL;
+			goto out_err;
+		}
+
+		DBG("Done formatting");
+		near_tag_set_blank(tag, FALSE);
+	}
+
+out_err:
+	return t1_cookie_release(err, cookie);
+}
+
+static int nfctype1_format(uint32_t adapter_idx, uint32_t target_idx,
+				near_tag_io_cb cb)
+{
+	int err;
+	struct near_tag *tag;
+	struct type1_cmd cmd;
+	struct t1_cookie *cookie;
+	uint8_t *uid, uid_length, addr;
+
+	DBG("");
+
+	tag = near_tag_get_tag(adapter_idx, target_idx);
+	if (tag == NULL)
+		return -EINVAL;
+
+	/* TODO: Dynamic tag format */
+	if (near_tag_get_memory_layout(tag) != NEAR_TAG_MEMORY_STATIC)
+		return -EOPNOTSUPP;
+
+	uid = near_tag_get_nfcid(adapter_idx, target_idx, &uid_length);
+	if (uid == NULL || uid_length != UID_LENGTH) {
+		near_error("Invalid type 1 UID");
+		return -EINVAL;
+	}
+
+	cookie = g_try_malloc0(sizeof(struct t1_cookie));
+	if (cookie == NULL) {
+		err = -EINVAL;
+		goto out_err;
+	}
+
+	cookie->adapter_idx = adapter_idx;
+	cookie->target_idx = target_idx;
+	cookie->cb = cb;
+	memcpy(cookie->uid, uid, UID_LENGTH);
+	cookie->cc[0] = TYPE1_MAGIC;
+	cookie->cc[1] = TYPE1_TAG_VER_1_1;
+	cookie->cc[2] = TYPE1_TAG_STATIC_SIZE_120;
+	cookie->cc[3] = TYPE1_READ_WRITE_ACCESS;
+	cookie->current_block = 1;
+	cookie->current_byte = 0;
+
+	cmd.cmd  = CMD_WRITE_E;
+	addr = cookie->current_block << 3;
+	cmd.addr = addr | (cookie->current_byte & 0x7);
+	cmd.data[0] = cookie->cc[cookie->current_byte];
+	cookie->current_byte++;
+	memcpy(cmd.uid, cookie->uid, UID_LENGTH);
+	g_free(uid);
+
+	err = near_adapter_send(cookie->adapter_idx, (uint8_t *) &cmd,
+				sizeof(cmd), format_resp, cookie,
+				t1_cookie_release);
 	if (err < 0)
-		goto out;
+		goto out_err;
 
 	return 0;
 
-out:
-	t1_cookie_release(cookie);
+out_err:
+	g_free(cookie);
+	g_free(uid);
 
 	return err;
 }
@@ -674,6 +804,7 @@ static struct near_tag_driver type1_driver = {
 	.read           = nfctype1_read,
 	.write          = nfctype1_write,
 	.check_presence = nfctype1_check_presence,
+	.format		= nfctype1_format,
 };
 
 static int nfctype1_init(void)
